// Code generated by execgen; DO NOT EDIT.
// Copyright 2019 The Cockroach Authors.
// Use of this software is governed by the Business Source License
// included in the file licenses/BSL.txt.
// As of the Change Date specified in that file, in accordance with
// the Business Source License, use of this software will be governed
// by the Apache License, Version 2.0, included in the file
// licenses/APL.txt.

package colexec

import (
	"context"

	"github.com/cockroachdb/cockroach/pkg/col/coldata"
	"github.com/cockroachdb/cockroach/pkg/sql/colexecbase"
	"github.com/cockroachdb/cockroach/pkg/sql/colmem"
	"github.com/cockroachdb/cockroach/pkg/sql/types"
)

// TODO(yuzefovich): add benchmarks.

// NewRowNumberOperator creates a new Operator that computes window function
// ROW_NUMBER. outputColIdx specifies in which coldata.Vec the operator should
// put its output (if there is no such column, a new column is appended).
func NewRowNumberOperator(
	allocator *colmem.Allocator, input colexecbase.Operator, outputColIdx int, partitionColIdx int,
) colexecbase.Operator {
	input = newVectorTypeEnforcer(allocator, input, types.Int, outputColIdx)
	base := rowNumberBase{
		OneInputNode:    NewOneInputNode(input),
		allocator:       allocator,
		outputColIdx:    outputColIdx,
		partitionColIdx: partitionColIdx,
	}
	if partitionColIdx == -1 {
		return &rowNumberNoPartitionOp{base}
	}
	return &rowNumberWithPartitionOp{base}
}

// rowNumberBase extracts common fields and common initialization of two
// variations of row number operators. Note that it is not an operator itself
// and should not be used directly.
type rowNumberBase struct {
	OneInputNode
	allocator       *colmem.Allocator
	outputColIdx    int
	partitionColIdx int

	rowNumber int64
}

func (r *rowNumberBase) Init() {
	r.Input().Init()
}

type rowNumberNoPartitionOp struct {
	rowNumberBase
}

var _ colexecbase.Operator = &rowNumberNoPartitionOp{}

func (r *rowNumberNoPartitionOp) Next(ctx context.Context) coldata.Batch {
	batch := r.Input().Next(ctx)
	n := batch.Length()
	if n == 0 {
		return coldata.ZeroBatch
	}

	rowNumberVec := batch.ColVec(r.outputColIdx)
	if rowNumberVec.MaybeHasNulls() {
		// We need to make sure that there are no left over null values in the
		// output vector.
		rowNumberVec.Nulls().UnsetNulls()
	}
	rowNumberCol := rowNumberVec.Int64()
	sel := batch.Selection()
	if sel != nil {
		for _, i := range sel[:n] {
			r.rowNumber++
			rowNumberCol[i] = r.rowNumber
		}
	} else {
		for i := range rowNumberCol[:n] {
			r.rowNumber++
			rowNumberCol[i] = r.rowNumber
		}
	}
	return batch
}

type rowNumberWithPartitionOp struct {
	rowNumberBase
}

var _ colexecbase.Operator = &rowNumberWithPartitionOp{}

func (r *rowNumberWithPartitionOp) Next(ctx context.Context) coldata.Batch {
	batch := r.Input().Next(ctx)
	n := batch.Length()
	if n == 0 {
		return coldata.ZeroBatch
	}

	partitionCol := batch.ColVec(r.partitionColIdx).Bool()
	rowNumberVec := batch.ColVec(r.outputColIdx)
	if rowNumberVec.MaybeHasNulls() {
		// We need to make sure that there are no left over null values in the
		// output vector.
		rowNumberVec.Nulls().UnsetNulls()
	}
	rowNumberCol := rowNumberVec.Int64()
	sel := batch.Selection()
	if sel != nil {
		for _, i := range sel[:n] {
			if partitionCol[i] {
				r.rowNumber = 0
			}
			r.rowNumber++
			rowNumberCol[i] = r.rowNumber
		}
	} else {
		for i := range rowNumberCol[:n] {
			if partitionCol[i] {
				r.rowNumber = 0
			}
			r.rowNumber++
			rowNumberCol[i] = r.rowNumber
		}
	}
	return batch
}
